
                        ;===    BRAQING BADD BALLOON     ===;
                        ;=== Keegan Covey, (C) 2018-2020 ===;
                        
    ; Endless-running (jumping?) game where you run a gauntlet of enemies and
    ; obstacles through the Martian desert.

    ;TV FORMAT : NTSC

    ;====OBJECTS===============================CONTROLS==========================;
    ; Player, seconds:           P0       ; Button:     Jump
    ; Monsters, minutes:         P1       ; Right:      Speed up
    ; Cactus/wall:               BL       ; Left:       Slow down
    ;                                     ; Up:         Higher jump
    ;                                     ; Down:       Lower jump
    ;====CHANGELOG===============================================================;
    ;   09/13/2017  ; -Created template source file
    ;   09/14/2017  ; -Show background and player positioned on screen 
    ;               ; -Attempting to have multiple objects onscreen at once
    ;   09/15/2017  ; -Good jump dynamics with delay, two line kernel,
    ;               ;  differently colored hat
    ;   09/18/2017  ; -Fine-tuned jump more including high and low jumps; button 
    ;               ;  controls jump now
    ;   09/19/2017  ; -Randomized the appearance of the cactus objects
    ;               ; -Cactuses are now random height
    ;   09/22/2017  ; -Altered syntax a little, added tables for some new enemies 
    ;               ;  that I'm gonna try to implement
    ;               ; -Added color table for player
    ;   09/25/2017  ; -Finally added P1 monster sprites
    ;               ; -Chooses between worm and muncher
    ;               ; -Neither one animates yet, but there is a generalized code
    ;               ;  by which other monsters could be added in this way, 
    ;               ;  controlling for speed, color, size, etc
    ;               ; -Worm is still floating head, and doesn't multiply properly
    ;               ; -Implemented animation, finally! :3
    ;               ; -Got "worm" effect to work, though there's a bunch of extra
    ;               ;  "zero" bytes on the muncher I'd like to get rid of :/
    ;   09/26/2017  ; -Worm rises out of ground as it gets near player
    ;               ; -Debugged a couple problems with blinking sprites
    ;               ; -Added a nice gradient to the "ground" area
    ;   09/27/2017  ; -Added "hop" cycle while character isn't jumping
    ;               ; -Animation delay for enemy sprites
    ;   09/28/2017  ; -Cacti are random colors
    ;               ; -Implemented onscreen timer as playfield, then rejiggered
    ;               ;   as sprite timer
    ;               ; -Added background box to counter
    ;   09/29/2017  ; -Level-up dynamic works, somewhat
    ;               ; -Added "birdoid" monster
    ;               ; -Removed ugly malfunctioning background box from counter
    ;   10/02/2017  ; -Added "eye" monster, including chasing behavior
    ;               ; -Improved level-up dynamic, reordered enemy appearances
    ;   10/03/2017  ; -Added "behemoth" monster
    ;   10/04/2017  ; -Made behemoth's "rising out of ground" effect
    ;   10/05/2017  ; -Added "radiation dude"
    ;               ; -Background colors change as player levels up
    ;   10/06/2017  ; -Added wall/rock variation of cactus which only appears
    ;               ;   after a certain level
    ;   10/07/2017  ; -Muncher multiplies, but only as two widely separated copies
    ;               ; -Improved odds tables mechanism
    ;               ; -Made input testing and player setup into subroutines
    ;   10/08/2017  ; -Started making title screen; introduced game active/
    ;               ;   inactive logic
    ;   10/10/2017  ; -Finished title screen
    ;               ; -Collision detection + lives implementation!
    ;   10/14/2017  ; -Reordered monster creation routines for a little more 
    ;               ;   efficiency and readability
    ;   10/22/2017  ; -Fixed 30-second flicker glitch and mnstrUnder glitch,
    ;               ;   improving some game mechanics
    ;   10/24/2017  ; -Fixed behemoth rising when player is dead
    ;               ; -Changed timing of player/monster positioning routine
    ;   10/25/2017  ; -Added separate "monster flags" variable for a couple extra
    ;               ;   monsters
    ;   10/26/2017  ; -Added Blue Boi monster
    ;   11/01/2017  ; -Finally have sound effects infrastructure in place
    ;   11/06/2017  ; -Replaced title screen with all-new one that effectively
    ;               ;   has its own kernel
    ;   04/16/2018  ; -Implemented behavior for when color/odds tables end, so
    ;               ;   game can continue 5ever
    ;               ; -Made behaviors of eye, behemoth, worm more interesting/
    ;               ;   challenging
    ;               ; -Deleted beautiful but extremely storage-intensive title
    ;               ;   graphics ;-;
    ;   04/22/2018  ; -Have added sound effects for every enemy, improved jumping
    ;               ;   sounds also
    ;               ; -Made new, much cheaper title screen
    ;               ; -Made eye yet more annoying, added "floating" behavior to
    ;               ;   birdoid
    ;   04/25/2018  ; -Eliminated "level table" system, replacing it with less
    ;               ;   flexible but MUCH more efficient leveling algorithm
    ;   05/01/2018  ; -267+43+29=339 bytes before rehashing monster frame system
    ;   05/02/2018  ; -17+225+267=509 bytes after: that's 170 bytes freed! :3
    ;               ;   Added "monster frame" tables and shared GetFrames sub-
    ;               ;   routine which makes setting frames much more efficient
    ;   11/14/2018  ; -Blue boi now has irregular, unpredictable jumps
    ;   11/12/2020  ; -Implemented basic ending screen, showing final time
    ;   11/13/2020  ; -Worm now goes down once it reaches top of screen
    ;               ;  Also turns around after player goes by. Cosmetic but fun
    ;               ;  Introduced "streak" system for collecting extra lives
    ;   11/14/2020  ; -Fleshed out flash, extra life effects
    ;   11/17/2020  ; -Made Random into a BRK routine, saving about 15 precious
    ;               ;   bytes of RAM
    ;   11/18/2020  ; -There is now a lives counter. Squeezed out a bunch of extra
    ;               ;   RAM by removing redundant declarations
    ;   11/19/2020  ; -Recovered a bunch more RAM by changing some SLEEP macros
    ;               ;   into delay loops
    ;               ; -Replaced some sky tables with an automated system for
    ;               ;   drawing the graphics instead
    ;============================================================================;

    .PROCESSOR  6502        ;well, technically 6507 ;3
    .INCLUDE    "vcs.h"     ;for TIA and RIOT registers' locations in memory
    .INCLUDE    "macro.h"   ;for some Atari-related macros
    
    ;===CONSTANTS===
    ;These aren't actually stored in ROM or RAM, but
    ;are named for programmer's convenience/easy
    ;editing and debugging
                
FPS             = 60            ;frames per second (used in calculating game
                                ; timer). On an NTSC television, this will ALWAYS
                                ; be equal to 60. If anyone wants to rejigger this 
                                ; for PAL or, god help you, SECAM then change this

LEVEL_TIME      = $10           ;time (seconds) between levelings up           
                                ; Must be in hex, since it's calculated as a BCD
                                
MAIN_HEIGHT     = 70            ;192 - 3 = 189 - 5 = 184 - 8 = 176 - 8 = 168
                                ;    - 30 = 138 / 2 = 69 [beavis and butthead]
                                ;    - 1 = 68, there's another WSYNC I forgot about

STREAK          = 15            ;How many monsters must be dodged for an extra life?

;Player flags--------------

PLRXPOS         = 33            ;player's X position onscreen

JUMP_ARC        = 16            ;total distance of standard jump arc
JUMP_DELAY      = 2             ;factor by which jumpCount is delayed
HOP_HEIGHT      = 3
DEFHOPDELAY     = 2             ;default hopping delay (at slow/normal speed)

;Game state flags----------

ACTIVE          = %10000000
DEAD            = %01000000
TITLE           = %00100000

;Object flags--------------
            
CACTUS          = %00000001     ;cactus flag
MONSTER         = %00000010     ;monster flag (note all ground-level monsters share
                                ; the same flag since we can only show one at a time)
;Monster flags-------------
                      
MUNCHER         = %10000000
WORM            = %01000000
BIRDOID         = %00100000
EYE             = %00010000
BEHEMOTH        = %00001000
RADDUDE         = %00000100
BLUEBOI         = %00000010

DISABLE_MNSTR   = %11111101

BEH_START       = 130           ;behemoth's starting X position
BBHOP_HEIGHT    = 9

;Colors of things----------

CACTUS_BASECOL  = $C0
ROCK_BASECOL    = $00
MNCHR_COL       = $6A
WORM_COL        = $58
BOID_COL        = $0E
EYE_COL         = $AE
BEH_COL         = $BE
RADDUDE_COL     = $EE
BLUEBOI_COL     = $8E

CTR_COL         = $0D           ;color of counter object
LIVES_COL       = $1E

;=====================================================================================
;VARIABLES - Allocate some symbolic locations in RAM
;=====================================================================================

     SEG.U vars     ;Begin variables segment: nothing is actually written to RAM at
    .ORG $80        ; this point, only that space is named and allocated
                    ;NOTE ALSO that RAM is zero-page, making accessing it faster :3
                    ;The space from $00-7F is reserved for the TIA registers, and
                    ; declared in vcs.h
                    
gameState       .DS 1
    
frames          .DS 1   ;standard binary, counts frames up to 1 second

seconds         .DS 1   ;BCD for seconds played
minutes         .DS 1   ;BCD for minutes played
;hours           .DS 1   ;BCD for the insane (not currently implemented)

onesDigit       .DS 2   ;Where to find selected digit at
tensDigit       .DS 2   ;+0 = seconds
                        ;+1 = minutes
                        ;+2 = hours, not currently implemented

level           .DS 1   ;current level (updates by time interval)
lives           .DS 1
streak          .DS 1   ;Counts number of monsters safely dodged

;score           .DS 3   ;6 BCD digits

secondsGfx      .DS 1
minutesGfx      .DS 1
;hoursGfx        .DS 1
livesGfx        .DS 2   ;For the lives counters (for up to 8 lives)

skyCol          .DS 6   ;holds sky color table per level
cloudCol        .DS 2   ;holds cloud color table per level
mtnCol          .DS 1   ;color of mountains per level
bakCol          .DS 1   ;color of background per level
groundCol       .DS 1   ;color of ground area per level

plrXpos         .DS 1   ;doesn't change much, but is needed for title display
plrYpos         .DS 1   ;player's Y (from bottom of screen to top of player)
plrHeight       .DS 1
plrDraw         .DS 1   ;measures from top against player height to determine when to
                        ; start drawing player
plrPtr          .DS 2   ;pointer to player graphic table
plrClrPtr       .DS 2   ;pointer to player color table

mnstrXpos       .DS 1   ;monster's X (from left)
mnstrYpos       .DS 1   ;monster's Y (from bottom of screen to top of monster)
mnstrHeight     .DS 1   ;height of monster sprite
mnstrDraw       .DS 1   ;uses same basic mechanism as plrDraw
mnstrPtr        .DS 2   ;pointer to monster graphic table
mnstrUnder      .DS 1   ;used for "worm neck" effect
mnstrFrame      .DS 1   ;counter for monster animation
mnstrSpeed      .DS 1   ;how much faster monster goes than cacti
mnstrSize       .DS 1   ;used with NUSIZ1 to maintain monster size
mnstrCol        .DS 1   ;color of monster
mnstrAnimHi     .DS 8   ;8 monster animation frames (higher bytes)
mnstrAnimLo     .DS 8   ;8 monster animation frames (lower bytes)

mnstrOdds       .DS 8   ;+0 = odds of ANY monster appearing
                        ;+1 = odds of rad dude appearing
                        ;+2 = odds of birdoid appearing
                        ;+3 = odds of muncher appearing
                        ;+4 = odds of crazy worm appearing
                        ;+5 = odds of eye appearing
                        ;+6 = odds of blue boi appearing
                        ;+7 = odds of behemoth appearing
cactusOdds      .DS 1   ;odds of a cactus appearing (MUST FOLLOW mnstrOdds
                        ; in RAM!!!)

animDelay       .DS 1   ;higher number means slower animation

;These values will never occur simultaneously: as such they share the same
;address

multMnchr               ;multiplication factor for munchers                        
;radDudeCol              ;Since his color changes frame by frame
bbHop                   ;Blue boi's hop factor
titleCol                ;color (per scanline) of title graphic
wormTmr                 ;calculates when worm will start rising
boidTmr                 ;delay in birdoid motion
                .DS 1 

;Same with these, although they occur simultaneous to some of the ones above

wormDir
bbHopHeight             ;so hop factor can be randomized with each jump      
boidInitYpos            ;Initial Y position of birdoid
titleBaseCol            ;color of top scanline of title (for the "rolling" effect)
                .DS 1

jump            .DS 1   ;used for calculating jumping
jumpHeight      .DS 1   ;height that player actually jumps
jumpCount       .DS 1   ;counter for slowing jump cycle

hop             .DS 1   ;used for calculating hopping
hopCount        .DS 1   ;counter for slowing hop cycle
hopDelay        .DS 1   ;factor by which hop is slowed

objSpeed        .DS 1   ;speed of oncoming objects

objFlags        .DS 1   ;Each bit will note if object is onscreen or not--note
                        ; constants above for which bit flags which object
                        
mnstrFlags      .DS 1   ;like objFlags, but specifically for which type of monster is
                        ;onscreen

cactusXpos      .DS 1   ;cactus' X position (from left)
cactusHeight    .DS 1   ;height of cactus (also line where cactus/ball object will be
                        ; activated)
cactusWidth     .DS 1
cactusCol       .DS 1   ;stores randomly generated cactus color

rand8           .DS 1   ;16 bits of random number generation
rand16          .DS 1   ;upper bits of random # generation...not necessary, but 
                        ; greatly improves randomization effect

delay           .DS 1   ;delay counter for certain actions

flash           .DS 1   ;timer used in screen flash effect

titleBakCol
temp            .DS 1   ;temporary variable for when one is needed
tempPtr         .DS 2   ;temporary pointer for when one is needed

SFXleft         .DS 1   ;left audio channel index
SFXright        .DS 1   ;right audio channel index

    .ECHO "----",[$100 - *]d , "/ 128 bytes of RAM remaining"   ;Assembler will help
                                                                ; me keep track of my
                                                                ; precious RAM

;=====================================================================================
;ROM SEGMENT - contains game code and various tables for graphics and stuff
;=====================================================================================
;INITIALIZE MEMORY - Clear RAM, including stack, and all registers (c. Andrew Davie)
;-------------------------------------------------------------------------------------

    .SEG code           ;begin code segment (what will be written to ROM)
    .ORG $F000          ;8K ROM starts at this address

            CLD         ;Making sure decimal mode flag is cleared

Reset:      LDX #0      ;X = 0
            TXA         ;A = 0
            TAY         ;Y = 0

Clear:      DEX         ;decrement X around until it's 0 again
            TXS         ;set stack pointer to X
            PHA         ;push 0 to stack
            BNE Clear   ;keep decrementing stack through entirety of memory until
                        ; everything including X is 0, and pointer is back at $FF

;-------------------------------------------------------------------------------------
;Once-only initialization
;-------------------------------------------------------------------------------------

            LDA #3
            STA lives

            LDA #STREAK
            STA streak      ;Initialize streak countdown
            
    ;The game resets from this point if lives = 0; any variables before this will
    ;remain constant BETWEEN games

GameOver:   
            ;STA flash
            ;STA mnstrSize
            
            STA CXCLR       ;clear all collision registers
            ;LDA #0
            ;STA gameState
                          
            LDA #TITLE
            ORA gameState
            STA gameState       ;by doing this, can have both TITLE and DEAD flags
                                ;set, signifying time to show Game Over screen   
            LDA #$FF
            STA cactusHeight    ;cactus height is -1 at start (show no cactus)
            STA mnstrYpos
            STA mnstrHeight

            ;LDA #3
            ;STA lives           ;start with 3 lives

            LDA #HOP_HEIGHT
            STA hop             ;initialize hop distance to default

            LDA INTIM           ;seed random number generator. Must be done outside
            STA rand8           ; of gameplay loop, or else this will be the same
            EOR #$FF            ; series of numbers every frame
            STA rand16
            
            JSR FetchOdds       ;set level for first 30 seconds of game
            JSR GetLevCols      ;set colors for first minute of game
            JSR CalcTimer

;=====================================================================================
;START OF FRAME - total frame must be 262 scanlines total
;=====================================================================================
;Vertical sync - 3 scanlines to signal TV to start a new frame
;-------------------------------------------------------------------------------------

Frame:      LDA #2
            STA WSYNC
            STA VSYNC   ;start vertical sync
            
            STA WSYNC   ;three WSYNCs are needed for vertical sync
            STA WSYNC   ;here's the second WSYNC
            LSR
            STA WSYNC   ;and 3 WSYNCs
            
                     
            ;STA WSYNC
            STA VSYNC   ;end vertical sync
            
            LDA #44
            STA TIM64T  ;Timer for vertical blank
                        ;EQUATION: scanlines = (timer * 64) / 76 
            ;LDA #0
            ;STA CTRLPF
            ;STA GRP0
            ;STA GRP1
            
                
;-------------------------------------------------------------------------------------
;Vertical Blank - 37 scanlines to wait while TV sets laser to top of screen
;-------------------------------------------------------------------------------------
 
    ;This section is all spaghetti code: I hate it, and I hope I can find a way to
    ; do all this dead/lives/etc testing without it being the garbled mess it is now.
    ; In the meantime, it does what it's supposed to

            LDA #2
            STA objSpeed    ;default speed of oncoming objects 
            
            LDA delay
            BNE NotReset    

            LDA SWCHB       ;load console switches
            LSR
            BCS NotReset    ;branch if reset switch isn't engaged
         
    ;Only when reset switch is thrown
            
NewGame:    LDA #3          ;reset to 3 lives at game start       
            STA lives
            
            LDA #30         ;30 frame delay before pressing reset switch again
            STA delay

            LDA #0          ;reset level, frame/timer counts on new game
            STA level
            STA frames
            STA seconds
            STA minutes
            
            LDA #ACTIVE
            STA gameState
            
            JSR GetLevCols

        ;Get enemy odds for first level
            
            LDA #<OddsTab0
            STA tempPtr
            LDA #>OddsTab0
            STA tempPtr+1
            
            LDY #ODDS           ;set Y to 0
FirstOdds:  LDA (tempPtr),y     ;fetch from selected odds table
            STA mnstrOdds,y     ;store into odds variables
            DEY
            BPL FirstOdds       ;must be BPL, so as to include 0
            
            
            LDY #SFX_HOP
            JSR SFXtrigger

    ;Execute from here each time player dies
            
Continue:   LDA #0
            STA objFlags
            STA mnstrFlags
            STA mnstrXpos
            STA mnstrYpos
            STA mnstrHeight
            STA cactusXpos
            STA jump
            STA mnstrSize
            STA mnstrUnder
            STA mnstrSpeed
            STA flash
            STA REFP1
            
            STA CXCLR       ;clear all collision registers

            LDA #ACTIVE
            STA gameState   ;set game state to active, dead to 0
            
            LDA #DEFHOPDELAY
            STA hopDelay        ;initialize hop delay factor to default
            
            LDA #STREAK     ;reset streak ;-;
            STA streak

            LDA #PLR_HEIGHT
            STA plrHeight
            SEC
            SBC #1
            STA plrYpos
            
            LDA #HOP_HEIGHT
            STA hop

            LDA #$FF
            STA cactusHeight
            
            JSR LifeGrafs
            
            ;LDY #SFX_HOP
            ;JSR SFXtrigger      ;play hopping sound
            
NotReset:   

            JSR DoPlayer    ;has to be a separate subroutine to keep branches in page
            JSR DoMnstr
            
            LDA delay
            BEQ ResDelay0   ;don't decrement delay if it's already 0
            DEC delay
ResDelay0:
    
            LDA gameState
            ASL             ;is game state active? (Shifts ACTIVE bit into carry slot)
            BCC Inactive    ;if not, skip to "inactive" section

            JSR DoCactus    ;Make/move cactus object
            JMP Active      ;skip inactive section
            
    ;Inactive: only executes if game state is inactive

Inactive:   LDA #DEAD       
            BIT gameState   ;is player dead?
            BEQ DoTitle     ;if not, set then game is inactive but player isn't dead;
                            ; skip ahead and set title on 
            LSR             ;is TITLE already set AND player is dead?
            BIT gameState
            BNE DoTitle
                        
            ;Dead player
            
            STA HMCLR       ;player is dead; clear horizontal move registers
            
            LDA delay
            BEQ Delay0      ;if delay=0, skip decrement and delay check
            ;DEC delay       ;if delay>0, decrement delay counter...
            JMP Active      ;...and skip input check
Delay0:     LDA INPT4       ;check if button is pressed
            BMI Active      ;if not, keep game paused
            LDA #20
            STA delay       ;keep from jumping when game resumes
            
            LDA lives       ;how many lives?
            BPL Continue    ;if >=0, branch up to continue game
            
            JMP GameOver    ;if lives <0, reset to title
            
Active:     ;STA WSYNC       ;HMOVE must always come RIGHT after WSYNC
            ;STA HMOVE       ;do fine horizontal positioning, set moving objects going

            ;LDA livesGfx
            ;STA GRP0
            ;LDA livesGfx+1
            ;STA GRP1
            LDX #1
            LDA #134
            JSR XPosObject
            ;STA RESP0
            DEX
            LDA #142
            JSR XPosObject
            STA WSYNC
            STA HMOVE
            LDA #LIVES_COL
            STA COLUP0
            STA COLUP1
            LDA skyCol
            STA COLUBK      ;color of sky
            BPL VertBlank   ;Skip turning on title
            ;.BYTE $2C
            
DoTitle:    ;JSR Title
            LDA #TITLE
            ORA gameState
            STA gameState
;            JMP OverBlank
            
    ;*** Any code in vertical blank goes above here ***

VertBlank:  LDA INTIM
            BNE VertBlank   ;little empty loop to check if we're at the end of the
                            ;timer yet
            LDX #1
            STA WSYNC       ;last WSYNC to start screen blanking in right spot
            STX VBLANK      ;End screen blanking, start drawing!
                
;-------------------------------------------------------------------------------------
;PICTURE - 192 scanlines, what will be visible onscreen
;-------------------------------------------------------------------------------------

            ;CLC
            LDA #TITLE
            BIT gameState       ;is TITLE activated?
            BEQ DoPicture       ;if not, skip ahead to picture
            ;LDA #DEAD           
            ASL
            BIT gameState       ;if so...is player also dead?
            BEQ DrawTitle
            JSR Endscreen       ;if so, do the endscreen instead of title
            ;.BYTE $2C
            JMP OverBlank
            
            
DrawTitle:  JSR Title           ;otherwise, go to title-drawing routine...
            JMP OverBlank       ;and then skip to Overblank
            ;.BYTE $2C
            
DoPicture:  JSR Picture         ;jump to draw primary gameplay frame

OverBlank:            
            LDA #2      ;Begin overscan blanking
            STA WSYNC
            STA VBLANK  ;vertical blank: cease drawing anything to screen

            LDA #36
            STA TIM64T  ;set overscan timer (see vertical blank timer for equation)
                
;-------------------------------------------------------------------------------------
;Overscan - 30 lines of blank to fit visible display to all screens
;-------------------------------------------------------------------------------------

            JSR SFXUpdate   ;advance sound effects, once per frame

            ;LDA #0
            ;STA CTRLPF
            ;STA GRP0        ;Clear player graphic

            LDA #1
            STA VDELP0      ;vertical delay for player, which is suddenly working for
                            ;some reason
            ;LDA gameState
            ;BEQ Overscan
            
            LDA flash       ;is the flash on?
            BEQ TestLives
            
    ;Flash effect
            
            DEC flash       ;subtract one from flash
            AND #%00001111  ;and every 16 frames...
            CMP #%00001000
            BNE TestLives
            JSR SFXoff      ;play extra life sound
            LDY #SFX_LIFE
            JSR SFXtrigger
            
TestLives:  LDA streak      ;has streak counted down to 0?
            BNE TestColls
            
    ;Extra life!!!
    
            JSR SFXoff
            LDY #SFX_LIFE
            JSR SFXtrigger
            INC lives
            JSR LifeGrafs
            LDA #STREAK
            STA streak
            LDA #64
            STA flash
            
    ;Collision detections

TestColls:  LDA #DEAD
            BIT gameState   ;is player dead?
            BNE NotHurt     ;only check collisions if not already dead
            
            LDA #%01000000
            BIT CXP0FB      ;BIT 6: player collides with ball (cactus)
            BNE Dead
            LDA #%10000000
            BIT CXPPMM      ;BIT 7: player collides with monster
            BEQ NotHurt

Dead:       LDA #DEAD
            STA gameState   ;set player to be dead
            DEC lives       ;subtract one life
            LDA #60
            STA delay       ;set input delay timer
            LDA #0
            STA flash
            
            JSR SFXoff      ;disable any sound effects currently playing
            LDY #SFX_OUCH
            JSR SFXtrigger
NotHurt:    JSR CalcTimer 
                     
    ;*** Code in overscan period goes above here ***

Overscan:   LDA INTIM       ;check if overscan timer has ended yet
            BNE Overscan 

            JMP Frame       ;start new TV frame

;=====================================================================================
;END OF FRAME            
;=====================================================================================
;SUBROUTINES
;=====================================================================================
;Subroutines for drawing visible gameplay frame, title screen, and endscrees are
; in a separate file for editing convenience
;-------------------------------------------------------------------------------------

    .INCLUDE "picture.asm"
    
;-------------------------------------------------------------------------------------
;All subroutines related to monsters are in here, for the sake of easy editing.
;-------------------------------------------------------------------------------------

    .INCLUDE "monsters.asm"
    
;-------------------------------------------------------------------------------------
    ;===FINE ADJUST TABLE===
    ;Works with XPosObject to position objects horizontally with
    ;pixel precision, which Atari normally can't do by itself due
    ;to ratio of TIA and processor cycle times
;-------------------------------------------------------------------------------------

FADJLEFT = [#FINEADJ - *]d
    .ECHO "----", #FADJLEFT, "bytes before FineAdjust"
    .ALIGN 256  ;table only works if it's at top of page

FINEADJ = *

FineAdjust  DC.B %01110000      ;left 7
            DC.B %01100000      ;left 6
            DC.B %01010000      ;left 5
            DC.B %01000000      ;left 4
            DC.B %00110000      ;left 3
            DC.B %00100000      ;left 2
            DC.B %00010000      ;left 1
            DC.B %00000000      ;no adjust
            DC.B %11110000      ;right 1
            DC.B %11100000      ;right 2
            DC.B %11010000      ;right 3
            DC.B %11000000      ;right 4
            DC.B %10110000      ;right 5
            DC.B %10100000      ;right 6
            DC.B %10010000      ;right 7
        
fineAdjustTable .EQU FineAdjust - %11110001 ;11110001 is -15
 
;-------------------------------------------------------------------------------------
;LivesGrafs
;-------------------------------------------------------------------------------------

LifeGrafs:  .SUBROUTINE

            LDY lives       ;set y to lives
            LDX #0          ;index for lives graphic
            TXA             ;set A 0 also
            STA livesGfx
            STA livesGfx+1  ;also initialize both graphic bytes to 0

.AddLife:   CLC             ;gotta clear this each time
            ROL
            ROL             ;move existing life over to next slot...
            ORA #%00000001  ;and add new one
            DEY             ;decrement lives counter
            BMI .Return     ;finish if no more lives
            STA livesGfx,x  ;otherwise store to current lives register
            BCC .AddLife    ;branch back if current lives register isn't full

            LDX #1          ;otherwise set index to 1 (second set of lives)
            LDA livesGfx+1  ;load in second register
            ORA #1          ;add a life...
            STA livesGfx+1  ;and update
            BCS .AddLife    ;and just keep going!

.Return:    RTS

;-------------------------------------------------------------------------------------
;DoPlayer: Fetches player animation frames, handles hopping, jumping, and death
; animation, and also deals with player input.  A very busy routine.
;-------------------------------------------------------------------------------------

DoPlayer:   .SUBROUTINE

    ;START OF DOPLAYER

            LDA gameState
            ASL             ;is game active?
            BCS .Active     ;if so, branch ahead; otherwise check if player is dead
            
            ASL             ;is player dead?
            BCS .Dead
            RTS
            
.Dead:      LDA #<(PlrFrmDead + PLR_HEIGHT-1)
            STA plrPtr
            LDA #>(PlrFrmDead + PLR_HEIGHT-1)
            STA plrPtr+1
            
            LDA #<(PlrColor + PLR_HEIGHT-1)
            STA plrClrPtr
            LDA #>(PlrColor + PLR_HEIGHT-1)
            STA plrClrPtr+1
            
            LDA PLRXPOS-3
            STA plrXpos
            
            JMP .SetPlayer
.Active:
.InitPlr:   LDA #0
            STA NUSIZ0
            
            LDA #PLRXPOS
            STA plrXpos
            
            LDA #<(PlrColor + PLR_HEIGHT-1)
            STA plrClrPtr
            LDA #>(PlrColor + PLR_HEIGHT-1)
            STA plrClrPtr+1
            
            CLC
            LDA #PLR_HEIGHT-1
            CMP plrYpos
            BCC .notUnder
            STA plrYpos
            LDA #0
            STA jump
            
.notUnder:
            LDA jump
            BNE .Input   ;if already jumping, then skip to forward/backward tests
            
;"HOPPING" CYCLE----------------
    
            LDA #<(PlrFrm1 + PLR_HEIGHT-1)
            STA plrPtr
            LDA #>(PlrFrm1 + PLR_HEIGHT-1)
            STA plrPtr+1
            
            INC hopCount        ;increment hop delay counter
            LDA hopDelay    
            CMP hopCount        ;has counter reached delay factor?
            BCS .Input          ;if not, skip to input testing

            LDA #0
            STA hopCount        ;otherwise, reset hop delay counter to 0
            
            LDA plrYpos
            CLC
            ADC hop             ;add "hop" to Y position
            STA plrYpos
            
            DEC hop             ;decrement hop distance (through negative)
            LDA hop
            ;CLC
            ;ADC #HOP_HEIGHT     ;when -HOP_HEIGHT + HOP_HEIGHT = 0
            CMP #-HOP_HEIGHT
            BNE .Input          ;if not, skip to input testing

            LDA #PLR_HEIGHT-1   ;otherwise, reset Y position...
            STA plrYpos
            LDA #HOP_HEIGHT     ;and hop
            STA hop
            
            LDY #SFX_HOP
            JSR SFXtrigger      ;play hopping sound
            
;GET INPUT-----------------------
    
.Input:     ;JSR TestInput
            LDA delay
            BNE .Return
            LDA jump
            BNE .TestRight   ;if already jumping, then skip to forward/backward tests

.TestBtn:   LDA INPT4           ;JOYSTICK BUTTON
            BMI .TestRight
            LDA #HOP_HEIGHT
            STA hop             ;reset hop
            LDA #PLR_HEIGHT-1
            STA plrYpos         ;reset player Y position
            LDA #(JUMP_ARC+1)   ;jump arc value must be odd number
            STA jump            ;store jump arc into jump
            
            LDY #SFX_JUMP
            JSR SFXtrigger

.TestUp:    LDA #%00010000      ;Is UP also pressed?
            BIT SWCHA
            BNE .TestDown        ;if not, check if down is pressed
            LDA jump
            CLC
            ADC #4              ;add to jump height if up is pressed
            STA jump

.TestDown:  ;LDA #%00100000      ;is DOWN also pressed?
            ;BIT SWCHA
            ;BNE .CalcJump
            ;LDA jump
            ;SEC
            ;SBC #2              ;subtract from jump height if down is pressed
            ;STA jump

.CalcJump:  LDA jump
            LSR                 ;divide jump arc distance by 2  
            STA jumpHeight      ;store into jumpHeight

.TestRight:  
            LDA #DEFHOPDELAY
            STA hopDelay        ;default hopping speed

            LDA #%10000000      ;RIGHT
            BIT SWCHA
            BNE .TestLeft
            INC objSpeed        ;increase speed of oncoming objects
            INC objSpeed        ;(by two)
            LDA #DEFHOPDELAY-1
            STA hopDelay        ;hop faster
                
.TestLeft:  LDA #%01000000      ;LEFT
            BIT SWCHA
            BNE .Return
            DEC objSpeed        ;decrease speed of oncoming objects by one
            ;LDA #DEFHOPDELAY+1
            ;STA hopDelay        ;hop slower
.Return:

;JUMP DYNAMICS-------------------

            LDA jump
            BEQ .SetPlayer       ;if jump is 0, skip jump processing        
            
.DoJump:    LDA #<(PlrFrmJmp + PLR_HEIGHT-1)    ;load jump animation frame
            STA plrPtr
            LDA #>(PlrFrmJmp + PLR_HEIGHT-1)    ;higher byte of same
            STA plrPtr+1 
            
            INC jumpCount       ;Increment jump delay counter
            LDA #JUMP_DELAY     ;have we reached the delay factor?
            BIT jumpCount       ;test if we have
            BEQ .SetPlayer       ;if not, don't do any jump logic
            LDA #0              
            STA jumpCount       ;otherwise, reset jump delay counter

            DEC jump            ;Reduce jump quotient by 1

            LDA plrYpos         ;start with current Y position
            CLC                 ;MUST CLEAR CARRY FLAG HERE!!!
            ADC jump            ;add jump quotient
            SEC
            SBC jumpHeight      ;then subtract height of player jump
            
            LDX SWCHA
            CPX #%11011111
            BNE .NoDown
            SBC #7
            ;DEC jump
            ;DEC jump
            ;INC jumpHeight
            ;INC jumpHeight
.NoDown:    
            STA plrYpos         ;...and finally, update player's Y position 

            LDA jump
            ;CLC
            ;ADC jumpHeight
            BNE .SetPlayer      ;has jump reached 0 now?
            
            LDY #SFX_HOP
            JSR SFXtrigger      ;play hopping sound on landing
            
;PLAYER ANIMATION---------------

.SetPlayer: LDA #MAIN_HEIGHT    ;start with height of playable area
            CLC
            ADC plrHeight       ;plus height of player...
            SEC
            SBC plrYpos         ;minus player position from bottom...
            STA plrDraw         ;gives us line to start drawing player
            
            LDA plrPtr          ;get loaded player animation frame
            SEC
            SBC plrYpos         ;subtract player's Y position
            STA plrPtr          ;update plrPtr
            ;LDA plrPtr
            BCS .SetPlrCol      ;branches based on 
            DEC plrPtr+1
            
.SetPlrCol: LDA plrClrPtr
            SEC
            SBC plrYpos
            STA plrClrPtr
            BCS .PCNoCarry
            DEC plrClrPtr+1
.PCNoCarry:
            
    ; END OF DOPLAYER
            
            RTS
         
;-------------------------------------------------------------------------------------
;XPosObject
;Handles fine X positioning of objects. A and X should be loaded BEFORE this sub-
;routine is called.
;-------------------------------------------------------------------------------------
                
XPosObject: .SUBROUTINE              ;A = X coordinate

            STA WSYNC               ;X = object to be moved
            SEC
.DivBy15:   SBC #15
            BCS .DivBy15

            TAY
            LDA fineAdjustTable,y
            STA HMP0,x              ;the x should be from 0 to 5, and will correspond
                                    ;to HMP0, HMP1, HMM0, HMM1, HMBL
            STA RESP0,x             ;Commit to draw sprite at this position
            
            STA WSYNC

            RTS   
               
;=====================================================================================
;TABLES and some RELATED SUBROUTINES
;=====================================================================================
;GRAPHIC TABLES
;-------------------------------------------------------------------------------------

    .INCLUDE "graphics.asm"
    
;-------------------------------------------------------------------------------------
;SOUND EFFECTS ROUTINES AND TABLES
;-------------------------------------------------------------------------------------

    .INCLUDE "sfx.asm"                  

;-------------------------------------------------------------------------------------
;LEVELING ROUTINES AND TABLES
;-------------------------------------------------------------------------------------

    .INCLUDE "leveling.asm"
    
;-------------------------------------------------------------------------------------
;Random
;8-16 BIT "random" number generation algorithm...basically just a bunch of BIT-
; shifting and bitwise operations to create simulated randomness
;Notice the RTI? The BRK IRQ is hard-coded to jump to Random, so we can save a few
; precious bytes for every call for a random number.
;-------------------------------------------------------------------------------------

Random:     .SUBROUTINE ;.SUBROUTINE means we can use temporary labels in this
                        ;section (e.g. .NoEor)
            LDA rand8
            LSR         ;shift lower 8 bits right
            ROL rand16
            
            BCC .NoEor
            EOR #$B4    ;if carry flag is set, flip some bits
.NoEor:     STA rand8   ;...and store into lower 8 bits
            EOR rand16

            RTI

;=====================================================================================
;ROM check and interrupt vectors
;=====================================================================================

    ;Assembler will tell us how much ROM is left
    .IF (* & $FF)
INTLEFT = [($FFFA - *)]d
;TOTALFREE = #INTLEFT
        .ECHO "----", #INTLEFT, "bytes before interrupt vectors"
        ;.ECHO "------", #TOTALFREE, "total bytes in cartridge"
    .ENDIF 

    ;Interrupt vectors: at program start and certain other events, these are the bytes
    ;the Atari looks at to decide where to begin execution
    
    .ORG $FFFA  ;at the very end of ROM
    
        .WORD Reset     ;NMI
        .WORD Reset     ;RESET
        .WORD Random    ;IRQ (BRK will jump to this location in memory)
