
;   LEVELING.ASM contains routines and tables connected with levelling up,
;   including level color tables and routines for loading them, and routine for
;   getting odds of monsters/cacti appearing in a level

;-------------------------------------------------------------------------------------
;LEVEL (COLOR) TABLES
;-------------------------------------------------------------------------------------
;Every couple of levels the background colors change.  The colors per level are
;stored here. Unlike other graphics/color tables, these are stored top to bottom. This
;method seems pretty storage-intensive, and is, but I wanted to have a progression of
;particular color schemes and this seemed to me the best way to do it

CTABLEFT = [#COLORTABS - *]d
    .ECHO "----", #CTABLEFT, "bytes before color tables"
    .ALIGN 256
COLORTABS = *

;00:00-01:30
;LEVEL 0-1-----------   Desert, day
Level1
SkyCol1     .BYTE $44
            .BYTE $46
            .BYTE $48
            .BYTE $4A
            .BYTE $4C
            .BYTE $4E
            
CloudCol1   .BYTE $0C
            .BYTE $0E
            
MtnCol1     .BYTE $34

BakCol1     .BYTE $42

COLORS      = * - Level1    ;Get size for all color tables

;01:30-02:30
;LEVEL 2-------------   Desert, dusk
Level2
SkyCol2     .BYTE $00
            .BYTE $02
            .BYTE $50
            .BYTE $02
            .BYTE $50
            .BYTE $52
            
CloudCol2   .BYTE $30
            .BYTE $1E
            
MtnCol2     .BYTE $60

BakCol2     .BYTE $50

;02:30-03:30
;LEVEL 3-------------   Desert, night
Level3
SkyCol3     .BYTE $00
            .BYTE $00
            .BYTE $80
            .BYTE $00
            .BYTE $80
            .BYTE $60
            
CloudCol3   .BYTE $02
            .BYTE $08
            
MtnCol3     .BYTE $00

BakCol3     .BYTE $00

;03:30-04:30
;LEVEL 4-------------   Deep night
Level4
SkyCol4     .BYTE $00
            .BYTE $00
            .BYTE $00
            .BYTE $00
            .BYTE $02
            .BYTE $04
            
CloudCol4   .BYTE $42
            .BYTE $1C
            
MtnCol4     .BYTE $00

BakCol4     .BYTE $00

;04:30-05:30
;LEVEL 5-------------   Foggy
Level5
SkyCol5     .BYTE $00
            .BYTE $02
            .BYTE $04
            .BYTE $06
            .BYTE $08
            .BYTE $0A
            
CloudCol5   .BYTE $02
            .BYTE $04
            
MtnCol5     .BYTE $50

BakCol5     .BYTE $A0

;05:30-06:30
;LEVEL 6-------------   Snowstorm
Level6
SkyCol6     .BYTE $04
            .BYTE $02
            .BYTE $06
            .BYTE $04
            .BYTE $02
            .BYTE $00
            
CloudCol6   .BYTE $02
            .BYTE $06
            
MtnCol6     .BYTE $0E

BakCol6     .BYTE $02

;06:30-07:30
;LEVEL 7-------------   Snowy, clear
Level7
SkyCol7     .BYTE $9E
            .BYTE $9E
            .BYTE $9E
            .BYTE $9C
            .BYTE $9A
            .BYTE $8A
            
CloudCol7   .BYTE $0E
            .BYTE $0C
            
MtnCol7     .BYTE $0E

BakCol7     .BYTE $a4

;07:30-08:30
;LEVEL 8-------------   Green
Level8
SkyCol8     .BYTE $9E
            .BYTE $9E
            .BYTE $9E
            .BYTE $9C
            .BYTE $9A
            .BYTE $8A
            
CloudCol8   .BYTE $0E
            .BYTE $0C
            
MtnCol8     .BYTE $F6

BakCol8     .BYTE $F2

;-------------------------------------------------------------------------------------
;FIRST-LEVEL ODDS TABLE
;-------------------------------------------------------------------------------------
;Hold odds that monsters or cacti will appear in the first level

;0:00-0:30 
;LEVEL 0-------------------------------     
OddsTab0            
            .BYTE #31       ;Any monster: 1/# odds
                .BYTE #128  ;rad dude
                .BYTE #0    ;birdoid    : 1/(255-#) odds 
                .BYTE #0    ;muncher    : 1/(255-#) odds
                .BYTE #0    ;worm
                .BYTE #0    ;eye
                .BYTE #0    ;blue boi
                .BYTE #0    ;behemoth

            .BYTE #15       ;cactus     : 1/# odds

;LEVEL 0 DEBUG TABLE (~equal odds of every monster for testing)  
        
;            .BYTE #1        ;any monster
;                .BYTE #206  ;rad dude
;                .BYTE #180  ;birdoid
;                .BYTE #144  ;muncher
;                .BYTE #108  ;worm
;                .BYTE #72   ;eye
;                .BYTE #36   ;blue boi
;                .BYTE #0    ;behemoth

;            .BYTE #1        ;cactus

ODDS = * - OddsTab0 ;number of entries in odds table

    ;There used to be odds tables for 16 more levels in here; I decided to retool the
    ;level-up system to calculate the odds mathemetically, offering less precise
    ;control over the leveling-up process but a lot more ROM (and infinite theoretical
    ;levels)
    
;-------------------------------------------------------------------------------------
;LevelUp
;Calculates player's current level every LEVEL_TIME seconds by subtracting LEVEL_TIME
;from the current time in seconds repeatedly until the result is 0, suggesting that
;the current time is divisible by LEVEL_TIME AND that it's time to level up.
;This routine assumes that it begins in decimal mode, and remains in decimal mode on
;return.
;-------------------------------------------------------------------------------------

LevelUp:    .SUBROUTINE

            SEC
.GetMod:    SBC #LEVEL_TIME ;how fast player levels up (in BCD!)
            BEQ .LevUp
            BMI .SkipLevUp
            BPL .GetMod

.LevUp:     INC level       ;level up!
            CLD             ;exit decimal mode
            JSR FetchOdds   ;Per level, get odds of things appearing
            LDA #%00000001  ;ensure we only call GetLevCols on even-numbered levels
            BIT level       
            BEQ .SkipLevUp
            JSR GetLevCols
            
.SkipLevUp: SED             ;aaaand back to decimal mode

            RTS

;-------------------------------------------------------------------------------------
;FetchOdds: Per difficulty level, generate odds of any monster/cactus appearing at a
; given moment.
;-------------------------------------------------------------------------------------

FetchOdds:  .SUBROUTINE
     
            LDA level
            BEQ .Return         ;if level=0, odds table already loaded
            CMP #12             ;leveling stops here
            BCS .Return         ;also true if level >= 30
            
            LDA #%00000011      ;every 8th level
            BIT level
            BNE .EachMnstr
            LDA mnstrOdds
            LSR                 ;divide previous mnstrOdds by 2
            CMP #3              ;odds never greater than 1/4
            BCC .EachMnstr
            STA mnstrOdds
            
.EachMnstr: LDX #1              ;start doing odds for each monster         
.OddsLoop:  LDA mnstrOdds,x
            BEQ .QuitOdds       ;if mnstrOdds = 0, leave loop for further instructions
            
            LDA #255            ;255
            SEC
            SBC mnstrOdds,x     ; -current odds
            LSR                 ; /2
            LSR                 ; /2
            CLC
            ADC mnstrOdds,x     ; +current odds
            STA mnstrOdds,x
            
.Next:      INX                 ;next mnstrOdds table index
            CPX #ODDS           ;are we at last element of table?
            BNE .OddsLoop       ;if not, keep going

.QuitOdds:  LDA level
            CMP #13
            BCS .Return         ;no new monsters from level 13 onwards
            CMP #9
            BEQ .Return         ;skip a couple levels before introducing behemoth
            AND #%00000001
            CMP #%00000001      ;new monster on level 1, 5, 9, etc.
            BNE .Return
            
            DEX
            LDA mnstrOdds,x
            LSR
            LSR
            INX
            STA mnstrOdds,x        
            
.Return:    RTS

;-------------------------------------------------------------------------------------
;GetLevCols
;-------------------------------------------------------------------------------------

GetLevCols: .SUBROUTINE
            
            LDA #<Level1    ;Load lower byte of Level1 table's address
            STA tempPtr
            LDA #>Level1    ;Load upper byte of Level1 table's address
            STA tempPtr+1
            LDA level
            LSR             ;divide level # by 2, to increment through table
            AND #%00000111  ;limit to 0-7
            TAX             ;transfer to index register x
            BEQ .FirstLev   ;For levels 0 and 1 (00:00-01:30)

.NextLevel: LDA tempPtr
            CLC
            ADC #COLORS     ;move forward on color table
            STA tempPtr     ;update tempPtr
            DEX
            BNE .NextLevel  ;do again until on current level (x=0)
            
.FirstLev:  LDY #COLORS
.SetCols:   LDA (tempPtr),y ;indirect addressing; dereference address stored at location
            STA skyCol,y    ;direct addressing; store dereferenced value into array
            DEY
            BNE .SetCols 
            
            LDA gameState
            BNE .Return     ;not at title
            
            LDA #$00        ;title background color
            STA bakCol       
            
.Return:    RTS
  