
;   PICTURE.ASM holds two subroutines, for drawing the main gameplay frame and the 
;   title screen.

;-------------------------------------------------------------------------------------
;Picture: Critical part of the kernel, which draws the 192 lines visible on the 
; screen.
;-------------------------------------------------------------------------------------

Picture:    .SUBROUTINE

    ;Start with sky area, which is mostly decorative although I will try to put the
    ; lives counter up here
            LDA #0
            STA CTRLPF

            STA WSYNC           ;1 line of clear sky

            LDY #4              ;playfield AND scanline index
            LDA flash
            BEQ .NoFlash1
            ADC #$1E
            .BYTE $2C
.NoFlash1: LDA cloudCol
            STA COLUPF          ;load cloud color
            LDA skyCol+1
            STA COLUBK          ;change sky color
                
.Sky:       LDA PF1Cloud,y      ;draw cloud in sky
            STA PF1
            STA WSYNC
            DEY
            BNE .Sky

            LDA skyCol+2
            STA COLUBK          ;change sky color
            LDA cloudCol+1
            STA COLUPF          ;bottom line of cloud a different color
            LDA PF1Cloud,y
            STA PF1
            STA WSYNC
            
            LDA skyCol+3
            STA COLUBK          ;change sky color

            LDA #0  
            STA PF1             ;PF0 is blank in top 8 rows  
            LDA flash
            BEQ .NoFlash2
            ADC #$1E
            .BYTE $2C            
.NoFlash2:  LDA mtnCol          ;Olympus Mons color
            STA COLUPF

            LDA #%00000001
            STA temp
            ORA cactusWidth
            STA CTRLPF          ;make playfield reflect, set cactus size

            
            
            LDX #8              ;8 rows of top mountain
            STA WSYNC           ;line of clear sky
            
            LDA skyCol+4
            STA COLUBK          ;change sky color...

.MtnTop:   
            
            LDA PF2Mtn,x        ;increment through mountain tables
            STA PF2
            LDA temp
            STA PF1 
            ASL
            ORA #%00000001
            STA temp
            STA WSYNC              
            DEX
            BNE .MtnTop
            
            ;New registers for bottom of mountain
            LDA #%11111111  ;PF1 AND PF2 registers are filled in
            STA PF1
            STA PF2

            LDA skyCol+5
            STA COLUBK          ;keep changin' that sky color
            LDX #7              ;8 rows of mountain bottom
            ;LDY #7              ;mountain table index  
            LDA livesGfx
            STA GRP0            
            LDA livesGfx+1
            STA GRP1            



.MtnBottom: LDA PF0Mtn,x        ;increment through "mountain" table
            STA PF0
            STA WSYNC
            DEX
            BNE .MtnBottom
            
            LDA #0              ;one last line of mountains with no lives counter
            STA GRP0
            STA GRP1
            STA WSYNC
            
            CLC
            LDA flash
            BEQ .NoFlash3
            ADC #$1E
            .BYTE $2C
.NoFlash3:  LDA bakCol
            STA COLUBK          ;Mars background color
            LDA #0
            STA PF0
            STA PF1
            STA PF2             ;disable all playfield graphics

            LDA cactusCol
            STA COLUPF          ;cactus color (ball color always = playfield color)
            
    ;Putting object movements here allows us to also put sprites in the sky
            
            LDA plrXpos
            LDX #0              ;corresponds to Player 0
            JSR XPosObject      ;Position player on X axis
            
            
            LDA mnstrXpos
            LDX #1              ;1 = PL1 object (monster)
            JSR XPosObject      ;position monster horizontally
            
            LDA cactusXpos
            LDX #4
            JSR XPosObject

            LDA mnstrSize
            STA NUSIZ1          ;set height of monsters
            LDA mnstrCol
            STA COLUP1
            
            STA WSYNC
            STA HMOVE

            LDY #MAIN_HEIGHT    ;y indexes scanline #, AND also which
                                ;line of sprite(s) to show

;ALL SPRITE DRAWING SHOULD OCCUR WITHIN THIS LOOP
;-------------------------------------------------------------------------------------
.Main:      LDA (plrClrPtr),y   ;load current line of player color table
            STA COLUP0          ;store to player color register
            LDA plrHeight
            SEC
            SBC #1
            DEC plrDraw         ;count down until at top of player
            CMP plrDraw         ;performs plrHeight-1 - plrDraw
            BCS .DrawPlr        ;is plrHeight-1 >= plrDraw?
            LDA #0              ;GRP0 will be set to 0
            .BYTE $2C           ;fancy way to skip only next instruction
                
.DrawPlr:   LDA (plrPtr),y      ;load in player graphic table

;LINE 1 of 2 LINE KERNEL--------

.Line1:     STA WSYNC
            STA GRP0            ;load in new line of player graphics  
            
            CPY cactusHeight    ;is y = cactusHeight?
            BNE .NoCactus       ;if so, enable cactus
            LDA #2
            STA ENABL           
.NoCactus:  LDA #MONSTER
            BIT objFlags 
            BEQ .NoMnstr        ;if there's no monsters then don't bother

            ;DEC mnstrDraw
            LDA mnstrDraw
            DEC mnstrDraw
            SEC
            SBC mnstrHeight
            BCC .DrawMnstr
            BMI .Below
.NoMnstr:   LDA #0
            .BYTE $2C           ;skip ahead to WSYNC
.Below:     LDA mnstrUnder
            .BYTE $2C           ;skip ahead to WSYNC           
.DrawMnstr: LDA (mnstrPtr),y

;LINE 2 OF 2 LINE KERNEL--------

.Line2:     STA WSYNC
            STA GRP1            ;load in new line of monster graphics

            DEY                 ;decrement dual-scanline counter
            BPL .Main           ;if y>0, do next scanline pair
;-------------------------------------------------------------------------------------

            LDA #$34
            STA COLUBK          ;ground (background) color
            LDA #0
            STA ENABL           ;disable ball (cactus)
            STA GRP0            ;player/monster graphics will not appear underground
            STA GRP1
            STA NUSIZ0
            STA NUSIZ1
            STA REFP1

            LDA #$44
            STA groundCol

            LDY #6
.Ground:    STA WSYNC

            LDA groundCol
            CMP #$40
            BCC .BaseCol
            DEC groundCol
            .BYTE $2C

.BaseCol:   LDA #$30
            STA COLUBK
            DEY
            BNE .Ground
            
            JSR Counter     ;draw counter object
            
            LDY #5
.UGround:   STA WSYNC       ;last visible rows of screen
            DEY
            BNE .UGround

            RTS
            
;-------------------------------------------------------------------------------------
;Title: Basically just a separate kernel for displaying the title screen, using 
; playfield registers 1 and 2 to display the title
;-------------------------------------------------------------------------------------

TTLMNSIZE   = PLR_HEIGHT*8          ;height of area of screen where title is drawn
TTLBRDSIZE  = ((192-TTLMNSIZE)/2)   ;height of top and bottom border areas

Title:      .SUBROUTINE

            LDA #$16
            STA COLUBK              ;black background (blackground?)
            INC titleBaseCol        ;need to start with a different color each frame
            LDA titleBaseCol
            STA titleCol            ;...and then titleCol will cycle it each scanline

            LDX #TTLBRDSIZE/2
.TopBord:   STA WSYNC
            ;INC titleCol
            ;LDA titleCol
            ;STA COLUBK
            DEX
            BNE .TopBord
            
            ;LDX #TTLMNSIZE
            LDX #PLR_HEIGHT
.Main:      ;LDA #4                 ;draw player sprite using PF0 and PF2 registers
            ;STA titleCtr
            LDY #8
.Line:      STA WSYNC
            INC titleCol
            LDA titleCol
            STA COLUP0
            LSR
            STA COLUPF
            LDA PlrFrm1,x
            ASL
            ASL
            ASL
            ASL
            STA PF2
            LDA PlrFrm1,x
            STA PF0
            SLEEP 13
            ;TXA
            ;LDX #4
            ;DEX
            ;BNE .delay
            ;TAX
            
            LDA #0
            STA PF0
            STA PF2
            STA VDELP0
            DEY
            BNE .Line
            DEX
            BNE .Main
            
            STA WSYNC
            LDA #%00000110  ;3 Bs
            STA NUSIZ0
            LDY #5
            NOP
.delay:     DEY
            BNE .delay
            STA RESP0

            LDY #10
.BtmBord1:  STA WSYNC
            DEY
            BNE .BtmBord1
            
            LDY #B_HEIGHT
.Letters:   
            LDA LetterB,y
            STA WSYNC
            STA GRP0
            ;INC titleCol
            ;LDA titleCol
            ;STA COLUP0
            LDA #0
            STA WSYNC
            STA GRP0
            
            ;STA GRP0
            DEY
            BPL .Letters
            
            LDY #20
.BtmBord2:  STA WSYNC
            DEY
            BNE .BtmBord2
            
            ;INC titleBaseCol	
            
            RTS                       

;-------------------------------------------------------------------------------------
;Endscreen: Another separate kernel for displaying the end screen, which will just
; display the final score.  Could I find some way to incorporate this into the above
; title screen kernel? Probably, but EHHHHH
;-------------------------------------------------------------------------------------

DIGHEIGHT   = 5*8
ENDBRDSIZE  = ((192-DIGHEIGHT)/2)-1

Endscreen:   .SUBROUTINE

            LDA #$56            ;set the background color 
            STA COLUBK

            LDX #ENDBRDSIZE
.TopBord:   STA WSYNC
            DEX
            BNE .TopBord
            
            LDA #CTR_COL    ;Display color
            STA COLUP0      ;seconds
            STA COLUP1      ;minutes
            
            LDA #7          ;make the counter graphics extra wide
            STA NUSIZ0
            STA NUSIZ1

            LDX #2          ;2
.delay1:    DEX             ;2
            BNE .delay1     ;3/2 for a delay of 11 cycles
            STA RESP1       ;set position of minute counter
            SLEEP 10
            STA RESP0       ;seconds come right after
            
            LDX #5          ;set counter for draw loop
            STA WSYNC       ;WSYNC right before .Draw to keep everything aligned

.Draw:      LDY tensDigit   ;get seconds' 10's digit offset
            LDA Digits,y    ;look up offset in table
            AND #$F0        ;isolate 10's digit
            STA secondsGfx  ;store a line of graphics table
            LDY onesDigit   ;now do this for the 1's place...
            LDA Digits,y
            AND #$0F
            ORA secondsGfx  ;adds 1's digit in without disrupting 10's digit
            STA secondsGfx
            
            INC onesDigit   ;for seconds...
            INC tensDigit   ;(do tens place too)
            STA GRP0
            
            LDY tensDigit+1 ;get minutes' 10's digit offset
            LDA Digits,y    ;look up offset in table
            AND #$F0        ;isolate 10's digit
            STA minutesGfx  ;store a line of graphics table
            LDY onesDigit+1 ;now do this for the 1's place...
            LDA Digits,y
            AND #$0F
            ORA minutesGfx  ;adds 1's digit in without disrupting 10's digit
            STA minutesGfx

            INC onesDigit+1 ;and for minutes
            INC tensDigit+1
      
            STA GRP1
            
            LDY #(DIGHEIGHT/5-1)
.Wsyncs     STA WSYNC
            DEY
            BNE .Wsyncs
            
            DEX   
            BNE .Draw       ;keep looping until x is 0

            ;STA WSYNC
            
            STX GRP0        ;set seconds digit graphics to 0
            STX GRP1        ;set minutes digit graphics to 0           
            
            LDX #ENDBRDSIZE
.BtmBord:   STA WSYNC
            ;INC titleCol
            ;LDA titleCol
            ;STA COLUBK
            DEX
            BNE .BtmBord
            
            ;INC titleBaseCol	
            
            RTS                       
                  
;-------------------------------------------------------------------------------------
;CalcTimer
;-------------------------------------------------------------------------------------

CalcTimer:  .SUBROUTINE 
                     
            LDA gameState
            ASL
            BCC .Stopped
            INC frames
.Stopped:   LDA frames      ;check if we're up to one second of frames
            CMP #FPS
            BCC NoTimeCalc  ;if not, skip to drawing timer
            LDA #0
            STA frames      ;otherwise, set to 0        
            
            SED             ;switch to decimal mode :O 

            LDA seconds
            CLC
            ADC #1          ;add one second to timer
            STA seconds
            
            JSR LevelUp     ;check every second if we're ready to level up yet
            
            LDA seconds
            CMP #$60        ;are we up to 60 seconds?
            BCC NoTimeCalc  ;if not, skip to drawing timer
            
            LDA #0
            STA seconds     ;reset seconds to 0
            LDA minutes
            CLC
            ADC #1          ;add 1 minute
            STA minutes
            ;CMP #$60        ;up to 60 minutes?
            ;BCC NoTimeCalc  ;if not, etc etc
            
            ;LDA #0
            ;STA minutes     ;reset minutes to 0
            ;LDA hours
            ;CLC
            ;ADC #1
            ;STA hours       ;add 1 hour (god damn though is this even necessary?)

    ;FETCH OFFSETS FOR TIMER GRAPHICS

NoTimeCalc: CLD             ;make sure we're no longer in decimal mode            
            LDX #1          ;decrement through mins, secs respectively
                            ;   seconds+0 = seconds (obvi)
                            ;   seconds+1 = minutes
TimerGrafs: LDA seconds,x
            AND #$0F        ;isolate 1's place
            STA temp
            ASL             ; X*2
            ASL             ;  *2
            ADC temp        ;  +X = X*5
            STA onesDigit,x ;where to find 1's digit
            LDA seconds,x
            AND #$F0        ;isolate 10's place
            LSR             ; X/2
            LSR             ;  /2
            STA temp
            LSR             ;  /2
            LSR             ;  /2
            ADC temp        ;  +(X/4) = X/16 + X/4 = X/16 + 4X/16 = 5*(X/16)
            STA tensDigit,x ;where to find 10's digit
            DEX
            BPL TimerGrafs

            RTS
            
;-------------------------------------------------------------------------------------
;Counter: Draws timer object at the bottom of the screen
;-------------------------------------------------------------------------------------

Counter:    .SUBROUTINE

            STA WSYNC
            
            LDA #CTR_COL    ;Display color
            STA COLUP0      ;seconds
            STA COLUP1      ;minutes

            LDX #11         ;2
.delay:     DEX             ;2
            BNE .delay      ;3/2 for a delay of 56 cycles
            
            STA RESP1       ;set position of minute counter
            STA RESP0       ;seconds come right after
            
            LDX #5          ;set counter for draw loop
            STA WSYNC       ;WSYNC right before .Draw to keep everything aligned

.Draw:      LDY tensDigit   ;get seconds' 10's digit offset
            LDA Digits,y    ;look up offset in table
            AND #$F0        ;isolate 10's digit
            STA secondsGfx  ;store a line of graphics table
            LDY onesDigit   ;now do this for the 1's place...
            LDA Digits,y
            AND #$0F
            ORA secondsGfx  ;adds 1's digit in without disrupting 10's digit
            STA secondsGfx
            
            INC onesDigit   ;for seconds...
            INC tensDigit   ;(do tens place too)
            STA GRP0
            
            LDY tensDigit+1 ;get minutes' 10's digit offset
            LDA Digits,y    ;look up offset in table
            AND #$F0        ;isolate 10's digit
            STA minutesGfx  ;store a line of graphics table
            LDY onesDigit+1 ;now do this for the 1's place...
            LDA Digits,y
            AND #$0F
            ORA minutesGfx  ;adds 1's digit in without disrupting 10's digit
            STA minutesGfx

            INC onesDigit+1 ;and for minutes
            INC tensDigit+1
      
            STA GRP1
            DEX   
            BNE .Draw       ;keep looping until x is 0

            STA WSYNC
            
            STX GRP0        ;set seconds digit graphics to 0
            STX GRP1        ;set minutes digit graphics to 0

            RTS

